/*
 *************
 * Copyright *
 *************
 *
 * Copyright (C) 2010 Christoph Zauner
 */

/*
 ***********
 * License *
 ***********
 *
 * This file is part of Rihamark.
 * 
 * Rihamark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Rihamark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Rihamark.  If not, see <http://www.gnu.org/licenses/>.
 */

package rmk.core;

import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import rmk.SPInterfaces.Algorithm;
import java.util.ServiceLoader;

public class AlgorithmService {

    private static AlgorithmService service;
    private ServiceLoader<Algorithm> loader;
	private ArrayList<Algorithm> algorithms = new ArrayList<Algorithm>();

    /**
     * Creates a new instance of this service.
     */
    private AlgorithmService() {
        loader = ServiceLoader.load(Algorithm.class);

		for(Algorithm algorithm: loader) {
			algorithms.add(algorithm);
		}

		Debug.debugPrint(Debug.LogLevel.DEBUG1, "Initial loading of the following algorithm service providers: " + algorithms);
    }

    /**
     * Retrieve the singleton static instance of DictionaryService.
     */
    public static synchronized AlgorithmService getInstance() {
        if (service == null) {
            service = new AlgorithmService();
        }
        return service;
    }

	/**
	 *
	 * @return ArrayList<String> containing the names of all found
	 * Algorithm Service Providers.
	 */
    public ArrayList<String> getSrvProvNames() {
        ArrayList<String> serviceProviderNames = new ArrayList<String>();

		for(Algorithm algorithm: algorithms) {
			serviceProviderNames.add(algorithm.getName());
		}

        return serviceProviderNames;
    }

	/**
	 *
	 * @return
	 */
    public Algorithm getSrvProv(int idxOfSrvProv) {
		return algorithms.get(idxOfSrvProv);
    }

	public Algorithm getSrvProvInstance(int idxOfSrvProv) {
		Algorithm newAlgorithm = null;

		try {
			newAlgorithm = algorithms.get(idxOfSrvProv).getClass().newInstance();
		} catch (InstantiationException ex) {
			Logger.getLogger(AlgorithmService.class.getName()).log(Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			Logger.getLogger(AlgorithmService.class.getName()).log(Level.SEVERE, null, ex);
		}

		return newAlgorithm;
	}

} // end of class
