/*
 *************
 * Copyright *
 *************
 *
 * Copyright (C) 2010 Christoph Zauner
 */

/*
 ***********
 * License *
 ***********
 *
 * This file is part of Rihamark.
 * 
 * Rihamark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Rihamark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Rihamark.  If not, see <http://www.gnu.org/licenses/>.
 */

package rmk.core;

/**
 *
 * @author christoph
 */
public class Debug {

private static LogLevel verbosity = LogLevel.DEBUG2;
public static enum LogLevel{ERR,WARN,INFO,DEBUG1,DEBUG2};

public static void debugPrint(String msg) {
	System.out.println(msg);
}

public static void debugPrint(LogLevel loglevel, Exception e, String s) {
	if(loglevel.compareTo(verbosity) > 0) {return;}

	switch(loglevel) {
		case ERR:    System.out.print("[ERR] ");    break;
		case WARN:   System.out.print("[WARN] ");   break;
		case INFO:   System.out.print("[INFO] ");   break;
		case DEBUG1: System.out.print("[DEBUG1] "); break;
		case DEBUG2: System.out.print("[DEBUG2] "); break;
	}

	System.out.println(s);
}

public static void debugPrint(LogLevel loglevel, String s) {
	if(loglevel.compareTo(verbosity) > 0) {return;}

	switch(loglevel) {
		case ERR:    System.out.print("[ERR] ");    break;
		case WARN:   System.out.print("[WARN] ");   break;
		case INFO:   System.out.print("[INFO] ");   break;
		case DEBUG1: System.out.print("[DEBUG1] "); break;
		case DEBUG2: System.out.print("[DEBUG2] "); break;
	}

	System.out.println(s);
}

public static void debugPrint(LogLevel loglevel, String origin, String s) {
	if(loglevel.compareTo(verbosity) > 0) {return;}

	switch(loglevel) {
		case ERR:    System.out.print("[ERR] ");    break;
		case WARN:   System.out.print("[WARN] ");   break;
		case INFO:   System.out.print("[INFO] ");   break;
		case DEBUG1: System.out.print("[DEBUG1] "); break;
		case DEBUG2: System.out.print("[DEBUG2] "); break;
	}
	System.out.print("in " + origin + ": ");
	System.out.println(s);
}

public String getCurrentMethodName() {
	StackTraceElement stackTraceElements[] = (new Throwable()).getStackTrace();
	return stackTraceElements[1].toString();
}


/*****************************/
/* getter and setter methods */
/*****************************/


/**
 * @return the verbosity
 */
public static LogLevel getVerbosity() {
	return verbosity;
}

/**
 * @param verbosity the verbosity to set
 */
public static void setVerbosity(LogLevel newVerbosity) {
	verbosity = newVerbosity;
}

} // end of class
