/*
 *************
 * Copyright *
 *************
 *
 * Copyright (C) 2010 Christoph Zauner
 */

/*
 ***********
 * License *
 ***********
 *
 * This file is part of Rihamark.
 * 
 * Rihamark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Rihamark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Rihamark.  If not, see <http://www.gnu.org/licenses/>.
 */

package rmk.core;

public class Task extends Thread {

	private Dispatcher dispatcher;
	private TestPlan testplan;
	// wants the user this task to be cancelled?
	private boolean cancelled = false;
	private final TaskReport taskCaller;

	public Task(TestPlan testplan, TaskReport taskCaller) {
		this.testplan = testplan;
		dispatcher = new Dispatcher(testplan, this);
		this.taskCaller = taskCaller;
	}

	/*
	 * Main task. Executed in background thread.
	 */
	@Override
	public void run() {
		Debug.debugPrint(Debug.LogLevel.DEBUG2,"Executing worker thread.");
		dispatcher.runTestplan();
		Debug.debugPrint(Debug.LogLevel.DEBUG2, "Worker thread is done.");

		if(getNumOfJobs() == getProgress()) {
			taskCaller.reportFinished(true);
		} else {
			taskCaller.reportFinished(false);
		}
	}

	/**
	 * Used by the Core to report changes in the progress to the UI.
	 *
	 * @param progress then umber of finished jobs
	 */
	public void reportProgress(int progress) {
		taskCaller.reportProgress(progress);
	}

	/**
	 *
	 * @return the number of finished jobs
	 */
	public int getProgress() {
		return dispatcher.finishedTasks;
	}

	public int getNumOfJobs() {
		return testplan.getNumOfJobs();
	}

	/**
	 * @return the cancelled
	 */
	public boolean isCancelled() {
		return cancelled;
	}

	/**
	 * @param cancelled the cancelled to set
	 */
	public void setCancelled(boolean cancelled) {
		this.cancelled = cancelled;
	}

} // end of class Task
