/*
 *************
 * Copyright *
 *************
 *
 * Copyright (C) 2010 Christoph Zauner
 */

/*
 ***********
 * License *
 ***********
 *
 * This file is part of Rihamark.
 * 
 * Rihamark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Rihamark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Rihamark.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * RihamarkAboutBox.java
 */

package rmk.gui;

import rmk.core.TestPlan;
import rmk.core.Test;
import rmk.core.Debug;
import java.io.File;
import java.io.IOException;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import org.jdesktop.application.Action;

/**
 * GUI for configuring test plan properties.
 *
 * @author christoph
 */
public class GuiTestplanProperties extends javax.swing.JDialog {
    RihamarkView caller;
	TestPlan testplan;
	protected DefaultComboBoxModel defaultComboBoxModel;

    public GuiTestplanProperties(java.awt.Frame parent, RihamarkView caller, TestPlan testplan) {
        super(parent);
		this.caller = caller;
		this.testplan = testplan;

		defaultComboBoxModel = new DefaultComboBoxModel(new String[] { "ASCII" });
		defaultComboBoxModel.setSelectedItem(testplan.resultFormat);

        initComponents();

		if(testplan.saveResult) {
			enableSaveResultElems(true);
			jCheckBoxSave.setSelected(true);
		}
    }

	/*
	 * Enables or disables the GUI components relevant for
	 * configuring the saving of the testplan result.
	 *
     * @param b a boolean value, where true enables the components and
     *          false disables them
	 */
	private void enableSaveResultElems(boolean b) {
		if(b) {
			jComboBoxSaveAs.setEnabled(true);
			jtextFieldPath.setEnabled(true);
			jButtonBrowse.setEnabled(true);
		} else
		{
			jComboBoxSaveAs.setEnabled(false);
			jtextFieldPath.setEnabled(false);
			jButtonBrowse.setEnabled(false);
		}
	}

    @Action public void closeAboutBox() {
        dispose();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonOk = new javax.swing.JButton();
        buttonCancel = new javax.swing.JButton();
        jCheckBoxSave = new javax.swing.JCheckBox();
        jComboBoxSaveAs = new javax.swing.JComboBox();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jtextFieldPath = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        jButtonBrowse = new javax.swing.JButton();
        jCheckBox1 = new javax.swing.JCheckBox("", testplan.isDeletePreviousResults());
        jLabel4 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(rmk.gui.RihamarkApp.class).getContext().getResourceMap(GuiTestplanProperties.class);
        setTitle(resourceMap.getString("title")); // NOI18N
        setModal(true);
        setName("aboutBox"); // NOI18N

        buttonOk.setText(resourceMap.getString("buttonOk.text")); // NOI18N
        buttonOk.setName("buttonOk"); // NOI18N
        buttonOk.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonOkActionPerformed(evt);
            }
        });

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(rmk.gui.RihamarkApp.class).getContext().getActionMap(GuiTestplanProperties.class, this);
        buttonCancel.setAction(actionMap.get("closeAboutBox")); // NOI18N
        buttonCancel.setText(resourceMap.getString("buttonCancel.text")); // NOI18N
        buttonCancel.setName("buttonCancel"); // NOI18N

        jCheckBoxSave.setText(resourceMap.getString("jCheckBoxSave.text")); // NOI18N
        jCheckBoxSave.setName("jCheckBoxSave"); // NOI18N
        jCheckBoxSave.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jCheckBoxSaveItemStateChanged(evt);
            }
        });

        jComboBoxSaveAs.setModel(defaultComboBoxModel);
        jComboBoxSaveAs.setEnabled(false);
        jComboBoxSaveAs.setName("jComboBoxSaveAs"); // NOI18N

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        jtextFieldPath.setEditable(false);
        jtextFieldPath.setText(testplan.resultPath.toString());
        jtextFieldPath.setEnabled(false);
        jtextFieldPath.setName("jtextFieldPath"); // NOI18N

        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setToolTipText(resourceMap.getString("jLabel3.toolTipText")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        jButtonBrowse.setText(resourceMap.getString("jButtonBrowse.text")); // NOI18N
        jButtonBrowse.setEnabled(false);
        jButtonBrowse.setName("jButtonBrowse"); // NOI18N
        jButtonBrowse.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonBrowseActionPerformed(evt);
            }
        });

        jCheckBox1.setText(resourceMap.getString("jCheckBox1.text")); // NOI18N
        jCheckBox1.setName("jCheckBox1"); // NOI18N
        jCheckBox1.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jCheckBox1ItemStateChanged(evt);
            }
        });

        jLabel4.setText(resourceMap.getString("jLabel4.text")); // NOI18N
        jLabel4.setName("jLabel4"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel3)
                                    .addComponent(jLabel1)
                                    .addComponent(jLabel2))
                                .addGap(117, 117, 117)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jComboBoxSaveAs, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                                        .addComponent(jtextFieldPath, javax.swing.GroupLayout.DEFAULT_SIZE, 242, Short.MAX_VALUE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(jButtonBrowse))
                                    .addComponent(jCheckBoxSave)
                                    .addComponent(jCheckBox1)))
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(buttonOk)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(buttonCancel)))
                        .addContainerGap())
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel4, javax.swing.GroupLayout.PREFERRED_SIZE, 178, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap(426, Short.MAX_VALUE))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jCheckBoxSave)
                        .addGap(8, 8, 8)
                        .addComponent(jComboBoxSaveAs, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jtextFieldPath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jButtonBrowse)))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel3)
                        .addGap(18, 18, 18)
                        .addComponent(jLabel1)
                        .addGap(18, 18, 18)
                        .addComponent(jLabel2)))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 182, Short.MAX_VALUE)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(buttonCancel)
                            .addComponent(buttonOk)))
                    .addComponent(jCheckBox1))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void buttonOkActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonOkActionPerformed
		testplan.saveResult = jCheckBoxSave.isSelected();
		testplan.resultFormat = (String)defaultComboBoxModel.getSelectedItem();
		testplan.resultPath = new File(jtextFieldPath.getText());
		testplan.setDeletePreviousResults(jCheckBox1.isSelected());
		dispose();
    }//GEN-LAST:event_buttonOkActionPerformed

	private void jButtonBrowseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonBrowseActionPerformed
		JFileChooser fc = new JFileChooser(jtextFieldPath.getText());
		fc.setDialogTitle("Choose a file where the testplan results will be saved");
		fc.setApproveButtonText("Choose");

		fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		int returnVal = fc.showOpenDialog(this);

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = fc.getSelectedFile();

			if(!file.exists()) {
				try {
					file.createNewFile();
				} catch (IOException ex) {
					GuiMessageDialog.show(this, "The file could not be created. The path is:\n" + file.getAbsolutePath(), JOptionPane.ERROR_MESSAGE);
				}
			} else if(!file.isFile()) {
					GuiMessageDialog.show(this, "The file choosen is not a normal file. The path is:\n" + file.getAbsolutePath(), JOptionPane.ERROR_MESSAGE);
			} else if(!file.canWrite()) {
					GuiMessageDialog.show(this, "The file choosen is not writable. The path is:\n" + file.getAbsolutePath(), JOptionPane.ERROR_MESSAGE);
			} else {
				if(file.length() != 0) {
					GuiMessageDialog.show(this, "Choosen file is not empty.\nRihamark will overwrite it when starting a testplan.\nThe path is:\n" + file.getAbsolutePath(), JOptionPane.WARNING_MESSAGE);
				}
				jtextFieldPath.setText(file.getAbsolutePath());
			}
		} else {
			Debug.debugPrint("User clicked the cancel button.");
		}
}//GEN-LAST:event_jButtonBrowseActionPerformed

	private void jCheckBoxSaveItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jCheckBoxSaveItemStateChanged
		if(jCheckBoxSave.isSelected()) {
			enableSaveResultElems(true);
		} else {
			enableSaveResultElems(false);
		}
	}//GEN-LAST:event_jCheckBoxSaveItemStateChanged

	private void jCheckBox1ItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jCheckBox1ItemStateChanged
		testplan.setDeletePreviousResults(jCheckBox1.isSelected());
	}//GEN-LAST:event_jCheckBox1ItemStateChanged
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton buttonCancel;
    private javax.swing.JButton buttonOk;
    private javax.swing.JButton jButtonBrowse;
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JCheckBox jCheckBoxSave;
    private javax.swing.JComboBox jComboBoxSaveAs;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JTextField jtextFieldPath;
    // End of variables declaration//GEN-END:variables

} // end of class
