/*
 *************
 * Copyright *
 *************
 *
 * Copyright (C) 2010 Christoph Zauner
 */

/*
 ***********
 * License *
 ***********
 *
 * This file is part of Rihamark.
 * 
 * Rihamark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Rihamark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Rihamark.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * RihamarkView.java
 *
 * This class is responsible for controlling the GUI and the backend.
 *
 * Because we don't want to implement our own ListModel we use the default
 * implementation the JDK provides (DefaultListModel). Therefore the GUI
 * does not use the data model of the backend for the testplan management
 * (e.g. of which tests does a testplan consist of).
 *
 * The GUI only uses one list model for each item type. 
 */

package rmk.gui;

import rmk.core.AttackService;
import rmk.core.AlgorithmService;
import rmk.SPInterfaces.Algorithm;
import rmk.core.TestPlan;
import rmk.core.Debug;
import rmk.core.Conservator;
import rmk.core.Dispatcher;
import javax.swing.JList;
import org.jdesktop.application.Action;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import java.io.File;
import java.util.LinkedList;
import javax.swing.DefaultListModel;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import rmk.SPInterfaces.Attack;
import rmk.SPMenu.UserInterface;

/**
 * The application's main frame.
 */
public class RihamarkView extends FrameView implements UserInterface {

    protected Dispatcher dispatcher;
    protected TestPlan testplan;

    private JDialog aboutBox;
    private JDialog addBox;
    DefaultListModel listModelTests = new DefaultListModel();
    DefaultListModel listModelAlgorithms = new DefaultListModel();
    DefaultListModel listModelAttacks = new DefaultListModel();
	private File testplanFile = new File("/tmp/rihamark_testplan.xml");

	private GuiProgress guiProgress;

	/*
	 * As long as a class variable references the main window
	 * the Swing Application Framework does not exit. The "Exit"
	 * button works, but the "Close" button in the window decoration
	 * bar does not.
	 */
	//private JFrame mainFrame;

    public RihamarkView(SingleFrameApplication app) {
        super(app);

		/*
		 * The usage of a ListDataListener is not neccessary because
		 * we use ValueChanged events of the View to monitor changes in
		 * the test list.
		 */

		/*
		listModelTests.addListDataListener(
			new ListDataListener() {
				public void contentsChanged(ListDataEvent e)
				{
				ERROR	Debug.debugPrint(Debug.LogLevel.DEBUG2, "test list model: contentsChanged");
				}

			public void intervalAdded(ListDataEvent e) {}
			public void intervalRemoved(ListDataEvent e) {}
			}
		);
		*/

		testplan = new TestPlan();
        initComponents();
		resetContentPanel();
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = RihamarkApp.getApplication().getMainFrame();
            aboutBox = new RihamarkAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        RihamarkApp.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jSplitPane1 = new javax.swing.JSplitPane();
        jPanel1 = new javax.swing.JPanel();
        jButton3 = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        listViewTests = new javax.swing.JList();
        jScrollPane2 = new javax.swing.JScrollPane();
        listViewAttacks = new javax.swing.JList();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jScrollPane3 = new javax.swing.JScrollPane();
        listViewAlgorithms = new javax.swing.JList();
        jButton6 = new javax.swing.JButton();
        jButton5 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        menuBar = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        jMenuItem3 = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JPopupMenu.Separator();
        javax.swing.JMenuItem itemExit = new javax.swing.JMenuItem();
        javax.swing.JMenu jMenuItem2 = new javax.swing.JMenu();
        jMenuItem4 = new javax.swing.JMenuItem();
        jMenuItem1 = new javax.swing.JMenuItem();
        jMenuItem5 = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JPopupMenu.Separator();
        itemStartTestplan = new javax.swing.JMenuItem();
        itemPropertiesOfTestplan = new javax.swing.JMenuItem();
        jMenu8 = MenuCreator.getMenu(this);
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();

        mainPanel.setName("mainPanel"); // NOI18N

        jSplitPane1.setDividerLocation(300);
        jSplitPane1.setName("jSplitPane1"); // NOI18N

        jPanel1.setName("jPanel1"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(rmk.gui.RihamarkApp.class).getContext().getResourceMap(RihamarkView.class);
        jButton3.setText(resourceMap.getString("jButton3.text")); // NOI18N
        jButton3.setName("jButton3"); // NOI18N
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        listViewTests.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("listViewTests.border.title"))); // NOI18N
        listViewTests.setModel(listModelTests);
        listViewTests.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        listViewTests.setName("listViewTests"); // NOI18N
        listViewTests.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                listViewTestsMouseClicked(evt);
            }
        });
        listViewTests.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                listViewTestsValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(listViewTests);

        jScrollPane2.setName("jScrollPane2"); // NOI18N

        listViewAttacks.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("listViewAttacks.border.title"))); // NOI18N
        listViewAttacks.setModel(listModelAttacks);
        listViewAttacks.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        listViewAttacks.setName("listViewAttacks"); // NOI18N
        listViewAttacks.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                listViewAttacksMouseClicked(evt);
            }
        });
        listViewAttacks.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                listViewAttacksValueChanged(evt);
            }
        });
        jScrollPane2.setViewportView(listViewAttacks);
        listViewAttacks.getAccessibleContext().setAccessibleName(resourceMap.getString("listViewAttacks.AccessibleContext.accessibleName")); // NOI18N

        jButton1.setText(resourceMap.getString("jButton1.text")); // NOI18N
        jButton1.setName("jButton1"); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jButton2.setText(resourceMap.getString("jButton2.text")); // NOI18N
        jButton2.setName("jButton2"); // NOI18N
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        jScrollPane3.setName("jScrollPane3"); // NOI18N

        listViewAlgorithms.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("listViewAlgorithms.border.title"))); // NOI18N
        listViewAlgorithms.setModel(listModelAlgorithms);
        listViewAlgorithms.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        listViewAlgorithms.setName("listViewAlgorithms"); // NOI18N
        listViewAlgorithms.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                listViewAlgorithmsMouseClicked(evt);
            }
        });
        listViewAlgorithms.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                listViewAlgorithmsValueChanged(evt);
            }
        });
        jScrollPane3.setViewportView(listViewAlgorithms);

        jButton6.setText(resourceMap.getString("jButton6.text")); // NOI18N
        jButton6.setName("jButton6"); // NOI18N
        jButton6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton6ActionPerformed(evt);
            }
        });

        jButton5.setText(resourceMap.getString("jButton5.text")); // NOI18N
        jButton5.setName("jButton5"); // NOI18N
        jButton5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton5ActionPerformed(evt);
            }
        });

        jButton4.setText(resourceMap.getString("jButton4.text")); // NOI18N
        jButton4.setName("jButton4"); // NOI18N
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE)
                            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE)
                            .addComponent(jScrollPane3, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE)
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(jButton5)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jButton6))
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(jButton1)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jButton2))
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(jButton3)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jButton4)))
                        .addContainerGap())
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel1, javax.swing.GroupLayout.DEFAULT_SIZE, 265, Short.MAX_VALUE)
                        .addGap(22, 22, 22))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel2, javax.swing.GroupLayout.DEFAULT_SIZE, 171, Short.MAX_VALUE)
                        .addGap(116, 116, 116))))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addGap(18, 18, 18)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 179, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton3)
                    .addComponent(jButton4))
                .addGap(18, 18, 18)
                .addComponent(jLabel2)
                .addGap(18, 18, 18)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 154, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton1)
                    .addComponent(jButton2))
                .addGap(18, 18, 18)
                .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 171, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton5)
                    .addComponent(jButton6))
                .addContainerGap())
        );

        jSplitPane1.setLeftComponent(jPanel1);

        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.LINE_AXIS));
        jSplitPane1.setRightComponent(jPanel2);

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 899, Short.MAX_VALUE)
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 725, Short.MAX_VALUE)
        );

        menuBar.setName("menuBar"); // NOI18N

        jMenu1.setText(resourceMap.getString("jMenu1.text")); // NOI18N
        jMenu1.setName("jMenu1"); // NOI18N

        jMenuItem3.setText(resourceMap.getString("jMenuItem3.text")); // NOI18N
        jMenuItem3.setName("jMenuItem3"); // NOI18N
        jMenuItem3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem3ActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItem3);

        jSeparator1.setName("jSeparator1"); // NOI18N
        jMenu1.add(jSeparator1);

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(rmk.gui.RihamarkApp.class).getContext().getActionMap(RihamarkView.class, this);
        itemExit.setAction(actionMap.get("quit")); // NOI18N
        itemExit.setName("itemExit"); // NOI18N
        jMenu1.add(itemExit);

        menuBar.add(jMenu1);

        jMenuItem2.setText(resourceMap.getString("jMenuItem2.text")); // NOI18N
        jMenuItem2.setName("jMenuItem2"); // NOI18N
        jMenuItem2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem2ActionPerformed(evt);
            }
        });

        jMenuItem4.setText(resourceMap.getString("jMenuItem4.text")); // NOI18N
        jMenuItem4.setName("jMenuItem4"); // NOI18N
        jMenuItem4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem4ActionPerformed(evt);
            }
        });
        jMenuItem2.add(jMenuItem4);

        jMenuItem1.setText(resourceMap.getString("jMenuItem1.text")); // NOI18N
        jMenuItem1.setName("jMenuItem1"); // NOI18N
        jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem1ActionPerformed(evt);
            }
        });
        jMenuItem2.add(jMenuItem1);

        jMenuItem5.setText(resourceMap.getString("jMenuItem5.text")); // NOI18N
        jMenuItem5.setName("jMenuItem5"); // NOI18N
        jMenuItem5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem5ActionPerformed(evt);
            }
        });
        jMenuItem2.add(jMenuItem5);

        jSeparator3.setName("jSeparator3"); // NOI18N
        jMenuItem2.add(jSeparator3);

        itemStartTestplan.setText(resourceMap.getString("itemStartTestplan.text")); // NOI18N
        itemStartTestplan.setName("itemStartTestplan"); // NOI18N
        itemStartTestplan.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                itemStartTestplanActionPerformed(evt);
            }
        });
        jMenuItem2.add(itemStartTestplan);

        itemPropertiesOfTestplan.setText(resourceMap.getString("itemPropertiesOfTestplan.text")); // NOI18N
        itemPropertiesOfTestplan.setName("itemPropertiesOfTestplan"); // NOI18N
        itemPropertiesOfTestplan.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                itemPropertiesOfTestplanActionPerformed(evt);
            }
        });
        jMenuItem2.add(itemPropertiesOfTestplan);

        menuBar.add(jMenuItem2);

        jMenu8.setText(resourceMap.getString("jMenu8.text")); // NOI18N
        jMenu8.setName("jMenu8"); // NOI18N
        menuBar.add(jMenu8);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        setComponent(mainPanel);
        setMenuBar(menuBar);
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Menu item choosen: Create a new testplan.
     *
     */
    private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed
		testplan = new TestPlan();
		reloadTestplan(testplan);
    }//GEN-LAST:event_jMenuItem1ActionPerformed

    /**
     * Button pressed: "Add a new Test".
     *
     * @param evt
     */
    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
	addTestItem();
    }//GEN-LAST:event_jButton3ActionPerformed

    /**
     * Button pressed: "Add a new Attack".
     *
     * @param evt
     */
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
		// if no test exists adding an attack is not possible
		if (testplan.getNumOfTests() == 0) return;

		JFrame mainFrame = RihamarkApp.getApplication().getMainFrame();
		addBox = new GuiAddAttack(mainFrame, this);
		addBox.setLocationRelativeTo(mainFrame);
		RihamarkApp.getApplication().show(addBox);
    }//GEN-LAST:event_jButton1ActionPerformed

    /**
     * Button pressed: "Add a new Algorithm".
     *
     * @param evt
     */
    private void jButton5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton5ActionPerformed
		// if no test exists adding an algorithm is not possible
		if (testplan.getNumOfTests() == 0) return;

		JFrame mainFrame = RihamarkApp.getApplication().getMainFrame();
		addBox = new GuiAddAlgorithm(mainFrame, this);
		addBox.setLocationRelativeTo(mainFrame);
        RihamarkApp.getApplication().show(addBox);
    }//GEN-LAST:event_jButton5ActionPerformed

    /**
     * Listener method for list selection changes.
     *
     * @param evt
     */
    private void listViewTestsValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_listViewTestsValueChanged
		// Fange den ersten Event ab (es werden ja bei jeder Aenderung zwei Events abgeschickt)
		if(evt.getValueIsAdjusting() == false) {
			Debug.debugPrint("function listViewTestsValueChanged(). New selected test has index: " + listViewTests.getSelectedIndex());
			if(listViewTests.getModel().getSize() != 0) {
				updateGuiForTest(listViewTests.getSelectedIndex());
			// reset the content panel
			} else {
				resetContentPanel();
			}
		}
    }//GEN-LAST:event_listViewTestsValueChanged

    private void listViewAttacksValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_listViewAttacksValueChanged
	// Fange den ersten Event ab (es werden ja bei jeder Aenderung zwei Events abgeschickt)
	if(evt.getValueIsAdjusting() == false) {
	    // ignore event if list was just deselected
	    if (listViewAttacks.getSelectedIndex() == -1) {
		return;
	    }
	    Debug.debugPrint("Selection of attack has changed. New selected attack has index: " + listViewAttacks.getSelectedIndex());
	    updateGuiForAttack();
	}
    }//GEN-LAST:event_listViewAttacksValueChanged

    private void itemStartTestplanActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_itemStartTestplanActionPerformed

		// if no task exists, running the testplan is not feasable
		if (testplan.getNumOfJobs() == 0) return;

		JFrame mainFrame = RihamarkApp.getApplication().getMainFrame();
		guiProgress = new GuiProgress(mainFrame, true, testplan);
		guiProgress.setLocationRelativeTo(mainFrame);
		
		RihamarkApp.getApplication().show(guiProgress);
    }//GEN-LAST:event_itemStartTestplanActionPerformed

    /**
     * Function to handle the case if a test is selected but the item selection
	 * does not change. This happens if the properties of an attack or
	 * algorithm are displayed and the corresponding test item is selected.
     * 
     * @param evt
     */
    private void listViewTestsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_listViewTestsMouseClicked
	// only update the the GUI for Test if at least one Test exists
	if(listViewTests.getModel().getSize() != 0) {
		Debug.debugPrint("function listViewTestsMouseClicked(). Selected test has index: " + listViewTests.getSelectedIndex());
	    updateGuiForTest(listViewTests.getSelectedIndex());
	}
    }//GEN-LAST:event_listViewTestsMouseClicked

    /**
     * Button clicked - remove test item.
     *
     * @param evt
     */
    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
	removeListItem(listViewAttacks, TestPlan.ItemType.ATTACK);
    }//GEN-LAST:event_jButton2ActionPerformed

	private void jButton6ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton6ActionPerformed
		removeListItem(listViewAlgorithms, TestPlan.ItemType.ALGORITHM);
	}//GEN-LAST:event_jButton6ActionPerformed

	private void listViewAlgorithmsValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_listViewAlgorithmsValueChanged
		// Fange den ersten Event ab (es werden ja bei jeder Aenderung zwei Events abgeschickt)
		if(evt.getValueIsAdjusting() == false) {
			// ignore event if list was just deselected
			if (listViewAlgorithms.getSelectedIndex() == -1) {
				return;
			}
			Debug.debugPrint("Selection of algorithm has changed. New selected algorithm has index: " + listViewAlgorithms.getSelectedIndex());
			updateGuiForAlgorithm();
		}
	}//GEN-LAST:event_listViewAlgorithmsValueChanged

	/* 
	 * Deal with case when switching between attack and algorithm selection
	 */
	private void listViewAttacksMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_listViewAttacksMouseClicked
		listViewAlgorithms.clearSelection();
	}//GEN-LAST:event_listViewAttacksMouseClicked

	/*
	 * Deal with case when switching between attack and algorithm selection
	 */
	private void listViewAlgorithmsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_listViewAlgorithmsMouseClicked
		listViewAttacks.clearSelection();
	}//GEN-LAST:event_listViewAlgorithmsMouseClicked

	private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem2ActionPerformed

	}//GEN-LAST:event_jMenuItem2ActionPerformed

	private void itemPropertiesOfTestplanActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_itemPropertiesOfTestplanActionPerformed
		JFrame mainFrame = RihamarkApp.getApplication().getMainFrame();
		addBox = new GuiTestplanProperties(mainFrame, this, testplan);
		addBox.setLocationRelativeTo(mainFrame);
		RihamarkApp.getApplication().show(addBox);
	}//GEN-LAST:event_itemPropertiesOfTestplanActionPerformed

	/**
	 * Save testplan.
	 *
	 * TODO improve "user safety"
	 *
	 * @param evt
	 */
	private void jMenuItem4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem4ActionPerformed
		JFileChooser fc = new JFileChooser(testplanFile.toString());
		fc.setDialogTitle("Choose a file to save the testplan to");
		fc.setApproveButtonText("Save");

		int returnVal = fc.showOpenDialog(RihamarkApp.getApplication().getMainFrame());

        if (returnVal == JFileChooser.APPROVE_OPTION) {
			testplanFile = fc.getSelectedFile();
	 		Conservator.saveTestplan(testplanFile, testplan);
		} else {
            Debug.debugPrint("User clicked the cancel button.");
        }
	}//GEN-LAST:event_jMenuItem4ActionPerformed

	/**
	 * Load testplan.
	 *
	 * TODO improve "user safety"
	 *
	 * @param evt
	 */
	private void jMenuItem5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem5ActionPerformed
		JFileChooser fc = new JFileChooser(testplanFile.toString());
		fc.setDialogTitle("Choose a file to load a testplan from");
		fc.setApproveButtonText("Load");

		int returnVal = fc.showOpenDialog(RihamarkApp.getApplication().getMainFrame());

        if (returnVal == JFileChooser.APPROVE_OPTION) {
			testplanFile = fc.getSelectedFile();
			testplan = Conservator.loadTestplan(testplanFile);
			reloadTestplan(testplan);
		} else {
            Debug.debugPrint("User clicked the cancel button.");
        }
	}//GEN-LAST:event_jMenuItem5ActionPerformed

	private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
		int testIdx = listViewTests.getSelectedIndex();
		removeListItem(listViewTests, TestPlan.ItemType.TEST);
	}//GEN-LAST:event_jButton4ActionPerformed

/**
 * 
 * @param evt
 */
	private void jMenuItem3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem3ActionPerformed
		JFrame mainFrame = RihamarkApp.getApplication().getMainFrame();
		GuiProperties guiProperties = new GuiProperties(mainFrame, this);
		guiProperties.setLocationRelativeTo(mainFrame);
		RihamarkApp.getApplication().show(guiProperties);
	}//GEN-LAST:event_jMenuItem3ActionPerformed

    public void addTestItem() {
		/* add new test item to backend */
		testplan.addTest();
		/* add new test item to list model */
		listModelTests.addElement(testplan.getTestsToRun().getLast().getTestName());

		updateGuiForTest(listModelTests.size() - 1);
		listViewTests.setSelectedIndex(listModelTests.size() - 1);
		Debug.debugPrint("New test added at index " + listViewTests.getSelectedIndex());
    }

    public void addAttackItem(int choosenAttack) {
		Debug.debugPrint("Adding attack with ID " + choosenAttack);

		if(choosenAttack < 0) return;

		int testIdx = listViewTests.getSelectedIndex();

		/* add new item to backend */
		LinkedList<Attack> attacks = testplan.getTestsToRun().get(testIdx).getAttacksToRun();
		attacks.add(AttackService.getInstance().getSrvProvInstance(choosenAttack));

		/* add new test item to list model */

		String attackName = testplan.getTestsToRun().get(testIdx).getAttacksToRun().getLast().getName();
		listModelAttacks.addElement(attackName);
		listViewAttacks.setSelectedIndex(listViewAttacks.getModel().getSize() - 1);
		// make sure that there is no selection in the list of algorithms
		Debug.debugPrint("Clearing the selection in listViewAlgorithms");
		listViewAlgorithms.clearSelection();
    }

	public void addAlgorithmItem(int choosenAlgorithm) {
		Debug.debugPrint("Adding algorithm with ID " + choosenAlgorithm);
		
		if(choosenAlgorithm < 0) return;

		int testIdx = listViewTests.getSelectedIndex();

		/* add new item to backend */

		LinkedList<Algorithm> algorithms = testplan.getTestsToRun().get(testIdx).getAlgorithmsToRun();
		Algorithm algorithm = AlgorithmService.getInstance().getSrvProvInstance(choosenAlgorithm);
		algorithms.add(algorithm);


		/* add new test item to list model */

		String algorithmName = testplan.getTestsToRun().get(listViewTests.getSelectedIndex()).getAlgorithmsToRun().getLast().getName();
		listModelAlgorithms.addElement(algorithmName);
		listViewAlgorithms.setSelectedIndex(listViewAlgorithms.getModel().getSize() - 1);
		// make sure that there is no selection in the list of attacks
		Debug.debugPrint("Clearing the selection in listViewAttacks");
		listViewAttacks.clearSelection();
    }

    /**
	 * Updates the GUI for the corresponding test. Updating the
	 * test list model when loading a new testplan is not covered
	 * here.
	 *
     * @param testidx Index of test to update GUI for.
     */
    public void updateGuiForTest(int testidx) {

		/*
		 * Do nothing if there are no items in the test list model but
		 * a "value changed" event is fired. This happens when loading an
		 * empty testplan.
		 */
		if(testidx == -1) return;

		/* update right panel */
		jPanel2.removeAll();
		jPanel2.add(new RihamarkTestControl(
			testplan.getTestsToRun().get(testidx), this
		));
		jPanel2.validate();

		/* update attack list model */
		listModelAttacks.clear();
		for (Attack attack: testplan.getTestsToRun().get(testidx).getAttacksToRun()) {
			listModelAttacks.addElement(attack.getName());
		}

		/* update algorithm list model */
		listModelAlgorithms.clear();
		for (int i=0; i < testplan.getTestsToRun().get(testidx).getAlgorithmsToRun().size(); i++) {
			listModelAlgorithms.addElement(testplan.getTestsToRun().get(testidx).getAlgorithmsToRun().get(i).getName());
		}
    }

	/**
	 * Displays the view corresponding to the just selected attack.
	 */
    public void updateGuiForAttack() {
		/* update right panel */
		jPanel2.removeAll();

		int testIdx = listViewTests.getSelectedIndex();
		int attackIdx = listViewAttacks.getSelectedIndex();

		Attack attack = testplan.getTestsToRun().get(testIdx).getAttacksToRun().get(attackIdx);
		jPanel2.add(new GuiTestItem(attack));
		jPanel2.validate();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem itemPropertiesOfTestplan;
    private javax.swing.JMenuItem itemStartTestplan;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton5;
    private javax.swing.JButton jButton6;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu8;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem3;
    private javax.swing.JMenuItem jMenuItem4;
    private javax.swing.JMenuItem jMenuItem5;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JPopupMenu.Separator jSeparator1;
    private javax.swing.JPopupMenu.Separator jSeparator3;
    private javax.swing.JSplitPane jSplitPane1;
    protected javax.swing.JList listViewAlgorithms;
    protected javax.swing.JList listViewAttacks;
    protected javax.swing.JList listViewTests;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    // End of variables declaration//GEN-END:variables

    /**
     * 1. Removes the item from the testplan.
     * 2. Removes the item from the GUI ListModel.
     * 3. Updates the selection (or display if the item
     *    removed was the last item).
     * 
     * @param jList2
     */
    private void removeListItem(JList jList, TestPlan.ItemType itemType) {
		DefaultListModel jListModel = (DefaultListModel)jList.getModel();
		int itemidx = jList.getSelectedIndex();

		Debug.debugPrint("Removing item type " + itemType);
		Debug.debugPrint("Removing item with index " + itemidx);

		// nothing selected
		if(itemidx == -1) {
			Debug.debugPrint("No item selected. Removing nothing.");
			return;
		}

		/*** list has no items ***/

		if(jListModel.size() == 0) {
			return;
		}

		/* 
		 * Additionally to removing the item from the backend and 
		 * the GUI ListModel the selection is updated.
		 * 
		 * If the last item is removed the view is updated to
		 * display the last selected test.
		 */

		/*** list has only one item ***/

		if(jListModel.size() == 1) {
			if(itemType == TestPlan.ItemType.ATTACK) {
				// update selection
				updateGuiForTest(listViewTests.getSelectedIndex());
				// remove the item from the backend
				testplan.getTestsToRun().get(listViewTests.getSelectedIndex()).getAttacksToRun().remove(itemidx);
			} else if(itemType == TestPlan.ItemType.ALGORITHM) {
				// update selection
				updateGuiForTest(listViewTests.getSelectedIndex());
				// remove the item from the backend
				testplan.getTestsToRun().get(listViewTests.getSelectedIndex()).getAlgorithmsToRun().remove(itemidx);
			// removing a test item
			} else {
				// remove the item from the backend
				testplan.getTestsToRun().remove(itemidx);			
			}

			// update the list model(s)
			jListModel.remove(itemidx);
			if(itemType == TestPlan.ItemType.TEST) {
				listModelAttacks.clear();
				listModelAlgorithms.clear();
			}

			return;
		}

		/*** list has at least two items ***/

		// first item selected
		if(itemidx == 0) {
			jList.setSelectedIndex(itemidx + 1);
		// item in between or last item selected
		} else {
			jList.setSelectedIndex(itemidx - 1);
		}
		// actually remove the item from the backend and the GUI ListModel
		if(itemType == TestPlan.ItemType.ATTACK) {
			testplan.getTestsToRun().get(listViewTests.getSelectedIndex()).getAttacksToRun().remove(itemidx);
		} else if (itemType == TestPlan.ItemType.ALGORITHM) {
			testplan.getTestsToRun().get(listViewTests.getSelectedIndex()).getAlgorithmsToRun().remove(itemidx);
		} else {
			testplan.getTestsToRun().remove(itemidx);
		}

		// update the list model
		jListModel.remove(itemidx);
    }

	/**
	 * Displays the view corresponding to the just selected algorithm.
	 */
	private void updateGuiForAlgorithm() {
		/* update right panel */
		jPanel2.removeAll();

		int testIdx = listViewTests.getSelectedIndex();
		int algorithmIdx = listViewAlgorithms.getSelectedIndex();

		Algorithm algorithm = testplan.getTestsToRun().get(testIdx).getAlgorithmsToRun().get(algorithmIdx);
		jPanel2.add(new GuiTestItem(algorithm));
		jPanel2.validate();
	}

	/**
	 * Load the given testplan. Can be used to create a new testplan
	 * (= load a newly created testplan object) or to load an already
	 * populated testplan (e.g. load a saved testplan).
	 *
	 */
	private void reloadTestplan(TestPlan testplan) {
		jPanel2.removeAll();
		jPanel2.repaint();
		listModelTests.clear();
		listModelAttacks.clear();
		listModelAlgorithms.clear();
		/*
		 * Neccessary if the reloading of a testplan
		 * does not trigger a change in the test view.
		 */
		resetContentPanel();

		// load case
		if(testplan.getTestsToRun().size() > 0) {
			Debug.debugPrint(Debug.LogLevel.DEBUG1, "Loading a testplan.");
			updateGuiForTest(0);

			/* update test list model */
			for (int i=0; i < testplan.getTestsToRun().size(); i++) {
				listModelTests.addElement(testplan.getTestsToRun().get(i).getTestName());
			}
			listViewTests.setSelectedIndex(0);
		}
	}

	private void resetContentPanel() {
		jPanel2.removeAll();
		jPanel2.add(new GuiContentPanelMessage("The testplan has no tests. Please add at least one test."));
		jPanel2.validate();
	}

	public JFrame getMainFrame() {
		return RihamarkApp.getApplication().getMainFrame();
	}

	public TestPlan getTestPlan() {
		return testplan;
	}

} // end of class
