/*
 *************
 * Copyright *
 *************
 *
 * Copyright (C) 2010 Christoph Zauner
 */

/*
 ***********
 * License *
 ***********
 *
 * This file is part of Rihamark.
 * 
 * Rihamark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Rihamark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Rihamark.  If not, see <http://www.gnu.org/licenses/>.
 */

package rmk.plugins.algorithms.pHash;

import org.simpleframework.xml.*;

import rmk.core.Debug;
import java.io.File;
import java.util.ArrayList;
import org.pHash.pHash;
import org.pHash.BMBImageHash;
import rmk.SPArguments.*;
import rmk.SPInterfaces.Algorithm;
import rmk.core.Hash;
import rmk.core.Test;
import rmk.core.BMBHash;

/**
 *
 * @author christoph
 */
public class Block extends Algorithm {

	private static String name = "Block Mean Value Based Image Hash";
	private static String bundleName = "pHash";
	private static String description =
			"Fast inter testing: Supported";

	/* start: service provider options */
	@Element private SpinnerInt method;
	@Element private SpinnerInt presetSizeX;
	@Element private SpinnerInt blockSizeX;
	/* end: service provider options */

	public Block() {
		super(name, bundleName, description, true);
		Debug.debugPrint(Debug.LogLevel.DEBUG2,"Constructor in " + this.getClass().getCanonicalName());

		if(!deserialization) {
			// argument
			method = new SpinnerInt(1, "Method", 0, 2, 1);
			addSPArgument(method);
			
			// argument
			presetSizeX = new SpinnerInt(256, "Preset size X", 0, 9999, 1);
			addSPArgument(presetSizeX);

			// argument
			blockSizeX = new SpinnerInt(16, "Block size X", 0, 9999, 1);
			addSPArgument(blockSizeX);
		}
	}

	@Override
	public Hash getHash(File file) {

		pHash.setBMBImageHashArgs(method.getValue(), presetSizeX.getValue(), blockSizeX.getValue());
		byte[] hashValue = pHash.getBMBImageHash(file).getHash();
		BMBHash hash = new BMBHash(hashValue);

		return hash;
	}

	@Override
	public double cmpHashes(Hash hashA, Hash hashB) {
		BMBImageHash phHashA = new BMBImageHash();
		BMBImageHash phHashB = new BMBImageHash();

		phHashA.setHash(((BMBHash)hashA).getHashValue());
		phHashB.setHash(((BMBHash)hashB).getHashValue());

		return pHash.getBMBImageHashDistance(phHashA, phHashB);
	}

	@Override
	public double cmpImages(File fileA, File fileB) {
		pHash.setBMBImageHashArgs(method.getValue(), presetSizeX.getValue(), blockSizeX.getValue());
		return pHash.getBMBImagesDistance(fileA.toString(), fileB.toString());
	}

} // end of class
