/*
 *************
 * Copyright *
 *************
 *
 * Copyright (C) 2010 Christoph Zauner
 */

/*
 ***********
 * License *
 ***********
 *
 * This file is part of Rihamark.
 * 
 * Rihamark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Rihamark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Rihamark.  If not, see <http://www.gnu.org/licenses/>.
 */

package rmk.plugins.algorithms.pHash;

import org.simpleframework.xml.*;

import rmk.core.Debug;
import java.io.File;
import org.pHash.pHash;
import org.pHash.RadialImageHash;
import rmk.SPInterfaces.Algorithm;
import rmk.core.Hash;
import rmk.core.RadialHash;
import rmk.SPArguments.*;

public class Radial extends Algorithm {

	/* start: mandatory service provider variables */
	private static String name = "Radial Variance Based Image Hash";
	private static String bundleName = "pHash";
	private static String description =
			"Fast inter testing: Supported\n\n" +
			"Note: The score returned by this algortihm measures the similarity. " +
			"So a value near to 0 means that the images are supposed to be different, " +
			"whereas a value near to 1 means that the images are likely to be the same." +
			"Most other implemented algorithms measure the distance or differnece between" +
			"two images. To enable the comparison the score of this algorithm is adapted" +
			"(distance score = 1 - similarity score).";
	/* end: mandatory service provider variables */

	/* start: service provider options */
	@Element private SpinnerDbl sigma;
	@Element private SpinnerDbl gamma;
	@Element private SpinnerInt angles;
	/* end: service provider options */

	public Radial() {
		super(name, bundleName, description, true);
		Debug.debugPrint(Debug.LogLevel.DEBUG2,"Constructor in " + this.getClass().getCanonicalName());

		if(!deserialization) {
			// argument
			sigma = new SpinnerDbl(1.0, "Sigma (radius) of the gaussian filter", 0.0, 100.0, 0.1);
			addSPArgument(sigma);

			// argument
			gamma = new SpinnerDbl(1.0, "Gamma correction", 0.0, 100.0, 0.1);
			addSPArgument(gamma);

			// argument
			angles = new SpinnerInt(180, "Number of angles to consider", 0, 360, 1);
			addSPArgument(angles);
		}
	}

	@Override
	public Hash getHash(File file) {

		pHash.setRadialImageHashArgs(sigma.getValue(), gamma.getValue(), angles.getValue());
		byte[] hashValue = pHash.getRadialImageHash(file).getHash();
		RadialHash hash = new RadialHash(hashValue);

		return hash;
	}

	@Override
	public double cmpHashes(Hash hashA, Hash hashB) {
		RadialImageHash phHashA = new RadialImageHash();
		RadialImageHash phHashB = new RadialImageHash();

		phHashA.setHash(((RadialHash)hashA).getHashValue());
		phHashB.setHash(((RadialHash)hashB).getHashValue());

		return  1.0 - pHash.getRadialImageHashDistance(phHashA, phHashB);
	}

	@Override
	public double cmpImages(File fileA, File fileB) {
		pHash.setRadialImageHashArgs(sigma.getValue(), gamma.getValue(), angles.getValue());
		return 1.0 - pHash.getRadialImagesDistance(fileA.toString(), fileB.toString());
	}

} // end of class
