/*
 *************
 * Copyright *
 *************
 *
 * Copyright (C) 2010 Christoph Zauner
 */

/*
 ***********
 * License *
 ***********
 *
 * This file is part of Rihamark.
 * 
 * Rihamark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Rihamark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Rihamark.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This class is used by all ImageJ based attack plugins to acutally
 * run ImageJ.
 *
 * TODO Files with upper-case file extensions are not saved
 * properly. E.g. save(image.JPG) results in a new file called
 * image.jpg.
 */

package rmk.plugins.attacks.imageJ;

import ij.IJ;
import ij.ImagePlus;
import ij.io.FileSaver;
import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import rmk.core.Debug;
import rmk.core.Debug.LogLevel;

/**
 *
 * @author christoph
 */
class ImageJ {

	private File image = null;
	private String cmd = "";
	private String args = "";

	/**
	 * Any ImageJ command may be applied. You can find out which
	 * commands to use and which arguments to give them by running
	 * the Plugins - Macros - Recorder, and manually calling ImageJ
	 * menu commands on an open image.
	 *
	 * @param image the image to operate on
	 * @param cmd ImageJ command to run
	 * @param args arguments for the command
	 */
    public ImageJ(File image, String cmd, String args) {
		this.image = image;
		this.cmd = cmd;
		this.args = args;
    }

    /**
     * Execute the attack.
     */
    public void run() {
		Debug.debugPrint(LogLevel.DEBUG2,"ImageJ command: " + cmd);
		Debug.debugPrint(LogLevel.DEBUG2,"ImageJ command args: " + args);

		ImagePlus imagePlus = IJ.openImage(image.toString());
		IJ.run(imagePlus, cmd, args);
		IJ.save(imagePlus, image.toString());
	}
	
} // end of class

