/*
 *************
 * Copyright *
 *************
 *
 * Copyright (C) 2010 Christoph Zauner
 */

/*
 ***********
 * License *
 ***********
 *
 * This file is part of Rihamark.
 * 
 * Rihamark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Rihamark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Rihamark.  If not, see <http://www.gnu.org/licenses/>.
 */

package rmk.plugins.attacks.imageJ;

import ij.IJ;
import ij.ImagePlus;
import ij.io.FileSaver;
import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import rmk.core.Debug;
import org.simpleframework.xml.*;
import rmk.SPArguments.SpinnerInt;
import rmk.SPInterfaces.Attack;

/**
 * Converts an image to the JPEG format, stores the result
 * in a temporary file, reads in the temporary file, saves
 * the image in it's original format into the "Attack
 * Image Folder" and deletes the temporary file.
 *
 */
public class JpegMangler extends Attack {

	/* start: mandatory service provider variables */
	private static String name = "JPEG Compression";
	private static String bundleName = "Rihamark Default Attack Plugins (using ImageJ)";
	private static String description =
			"Converts an image to the JPEG image format, stores the result " +
			"in a temporary file, reads in the temporary file, saves " +
			"the image in it's original image format into the \"Attack " +
			"Image Folder\" and deletes the temporary file.";
	/* end: mandatory service provider variables */

	/* start: service provider options */
	@Element private SpinnerInt quality = new SpinnerInt(50, "JPEG Quality (0 - 100)", 0, 360, 1);
	/* end: service provider options */

	public JpegMangler() {
		super(name, bundleName, description);
		Debug.debugPrint(Debug.LogLevel.DEBUG2,"Constructor in " + this.getClass().getCanonicalName());

		if(!deserialization) {
			// argument
			addSPArgument(quality);
		}
	}

    public void runAttack(String image){
	    Debug.debugPrint("Running attack: " + getName());

		try {
			File temp = File.createTempFile("rmk_", null);

			ImagePlus imagePlus = IJ.openImage(image.toString());
			FileSaver fileSaver = new FileSaver(imagePlus);
			FileSaver.setJpegQuality(quality.getValue());
			fileSaver.saveAsJpeg(temp.getAbsolutePath());

			imagePlus = IJ.openImage(temp.getAbsolutePath());
			IJ.save(imagePlus, image);

			temp.delete();
		} catch (IOException ex) {
			Logger.getLogger(ImageJ.class.getName()).log(Level.SEVERE, null, ex);
		}
	}


	/*****************************/
	/* getter and setter methods */
	/*****************************/


} // end of class
