/*
 *************
 * Copyright *
 *************
 *
 * Copyright (C) 2010 Christoph Zauner
 */

/*
 ***********
 * License *
 ***********
 *
 * This file is part of Rihamark.
 * 
 * Rihamark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Rihamark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Rihamark.  If not, see <http://www.gnu.org/licenses/>.
 */

package rmk.plugins.attacks.imageJ;

import ij.IJ;
import ij.ImagePlus;
import java.io.File;
import rmk.core.Debug;
import org.simpleframework.xml.*;
import rmk.SPArguments.ComboBox;
import rmk.SPArguments.SpinnerInt;
import rmk.SPInterfaces.Attack;

/**
 * ImageJ implements the resizing logic in the GUI.
 * Therefore we have to reimplement it.
 *
 * ImageJ uses the floor function when calculating the new
 * size of an image (if one dimension is adjusted
 * proportionally). Gimp uses the round function.
 */
public class Resize extends Attack {

	/* start: mandatory service provider variables */
	private static String name = "Resize";
	private static String bundleName = "Rihamark Default Attack Plugins (using ImageJ)";
	private static String description = "Self explanatory.";
	/* end: mandatory service provider variables */

	/* start: service provider options */
	@Element private SpinnerInt width;
	@Element private SpinnerInt height;
	@Element private ComboBox interpolation;
	@Element private ComboBox adjust;
	/* end: service provider options */

	private ImageJ attack;

	public Resize() {
		super(name, bundleName, description);
		Debug.debugPrint(Debug.LogLevel.DEBUG2,"Constructor in " + this.getClass().getCanonicalName());

		if(!deserialization) {
			// argument
			width = new SpinnerInt(800, "Width (pixels)", 0, 10000, 1);
			addSPArgument(width);

			// argument
			height = new SpinnerInt(600, "Height (pixels)", 0, 10000, 1);
			addSPArgument(height);

			// argument
			interpolation = new ComboBox("None", "Interpolation", new String[] {"None", "Bilinear", "Bicubic"});
			addSPArgument(interpolation);

			// argument
			adjust = new ComboBox("None", "Adjust Proporionally", new String[] {"None", "Width", "Height"});
			addSPArgument(adjust);
		}
	}

    public void runAttack(String file){
	    Debug.debugPrint(Debug.LogLevel.DEBUG1, "Running attack: " + getName());

		int argWidth = width.getValue();
		int argHeight = height.getValue();
		
		if(adjust.getValue().equals("Width")) {
			ImagePlus imagePlus = IJ.openImage(file);
			double factor = (double)imagePlus.getHeight() / (double)argHeight;
			argWidth = (int) (imagePlus.getWidth() / factor);
		} else if(adjust.getValue().equals("Height")){
			ImagePlus imagePlus = IJ.openImage(file);
			double factor = (double)imagePlus.getWidth() / (double)argWidth;
			argWidth = (int) (imagePlus.getHeight() / factor);
		}

	    String args = String.format(
			"width=%d height=%d interpolation=%s",
		    argWidth,
		    argHeight,
		    interpolation.getValue()
	    );

		attack = new ImageJ(new File(file), "Size...", args);
		attack.run();
	}

	
	/*****************************/
	/* getter and setter methods */
	/*****************************/


} // end of class

